@@ -4,9 +4,32 @@
 #include <ctype.h>
 #include "tool.h"
 #include "link.h"
+#include <sys/types.h>
+#include <fcntl.h>
+#include <unistd.h>
 
 #define ALLOC_MODULE	5000
+ 
+ /*
+  * Check if the module is an a.out module, so that we can skip
+  * the first '_'. We now use a unified ELF-like symbol table format
+  */
+static int is_a_out(const char *objname) // 1 == a.out, 0 == ELF
+{
+	int fd;
+	char buf[4];
+	int a_out = 1; //if open fails, the following call to mod_open fails too
 
+	if ((fd = open(objname, O_RDONLY)) > 0) {
+		if (read(fd, buf, 4) == 4) {
+			if (buf[0] == 0177 && strncmp(buf +1, "ELF", 3) == 0)
+				a_out = 0;
+		}
+		close(fd);
+	}
+
+	return a_out;
+}
 
 /*
 	Ouvre le fichier et retourne NULL si erreur.
@@ -17,10 +40,10 @@ static FILE *module_open (const char *objname)
 	struct stat buf;
 	if (stat (objname,&buf) != -1){
 		char cmd[300];
-		sprintf (cmd,"nm -pg %s",objname);
+		sprintf (cmd,"nm -p %s",objname);
 		ret = popen_err (cmd,"r",0);
 	}else{
-		fprintf (stderr,"%s does not exist\n",objname);
+		depmod_error ("%s does not exist",objname);
 	}
 	return ret;
 }
@@ -94,6 +117,7 @@ PUBLIC MODULE *MODULES::setdummy (const char *name)
 PUBLIC int MODULES::loadobj(SYMBOLS &syms, const char *objname)
 {
 	int ret = -1;
+	int a_out = is_a_out(objname); // 1 == a.out, 0 == ELF
 	FILE *fin = module_open (objname);
 	if (fin != NULL){
 		MODULE *mod = tbmod+nbmod++;
@@ -109,7 +133,8 @@ PUBLIC int MODULES::loadobj(SYMBOLS &syms, const char *objname)
 		while (fgets(rbuf,sizeof(rbuf)-1,fin)!=NULL){
 			char buf[200];
 			str_strip (rbuf,buf);
-			char *name = buf+11;
+			// skip leading '_' for a.out modules
+			char *name = buf+11 + a_out;
 			if (buf[0] == ' '){
 				tbext[nbext++] = syms.add (name,NULL,SYM_REQUIS
 					,module_requis,0);
@@ -301,7 +326,7 @@ PUBLIC int MODULES::findundef (FILE *fout)
 			for (int e=0; e<nbext; e++, ptext++){
 				if (!(*ptext)->defini){
 					if (ret == 0){
-						fprintf (stderr,"Undefined symbols:\n");
+						fprintf (fout,"Undefined symbols:\n");
 					}
 					showundef (*ptext,fout);
 					(*ptext)->defini = 1;	// Evite de rementionner
@@ -342,7 +367,8 @@ PUBLIC void MODULES::showall (FILE *fout)
 		fprintf (fout,"%s %d\n",ptmod->name,ptmod->is_load);
 		SYMBOL **ptpub = ptmod->pub.tb;
 		int nbpub = ptmod->pub.nb;
-		for (int e=0; e<nbpub; e++, ptpub++){
+		int e;
+		for (e=0; e<nbpub; e++, ptpub++){
 			SYMBOL *ptsym = *ptpub;
 			fprintf (fout,"\tT %s %d %d %d %d %d\n",ptsym->name,ptsym->requis
 				,ptsym->defini,ptsym->vue_avant
